
' Generate a Pov-Ray "density file" (df3)
' "***" denotes lines or delimits sections which should
' be adjusted to taste or necessity

CONST TRUE = -1
CONST FALSE = NOT TRUE
CONST pi = 3.14159265358979#

'''''''''''''''''''''''''''''''''''''''''''''''''
' Define our density as a function of x,y,z coordinates

' ***-
A = .5
' -***

DEF fnDensity (x, y, z)
' ***-
    fnDensity = COS(A * pi * z) * COS(A * pi * y) + COS(A * pi * x) * COS(A * pi * z) + COS(A * pi * x) * COS(A * pi * y)
' -***
END DEF

'''''''''''''''''''''''''''''''''''''''''''''''''

'''''''''''''''''''''''''''''''''''''''''''''''''
' the name of the 'df3' file to write
' ***-
datafile$ = "\Progra~1\POV-Ra~1.1\dftest.df3"
' -***

'''''''''''''''''''''''''''''''''''''''''''''''''
' x, y, znd z resolutions
' ***-
xres = 30
yres = 30
zres = 30
' -***

'''''''''''''''''''''''''''''''''''''''''''''''''
' We are computing densities throughout a box.
' POV-Ray assumes the box is <0,0,0>,<1,1,1>
' The box we are actually computing with this
' program is <xmin,ymin,zmin>,<xmax,ymax,zmax>
' ***-
xmin = -pi: xmax = pi
ymin = -pi: ymax = pi
zmin = -pi: zmax = pi
' -***


'''''''''''''''''''''''''''''''''''''''''''''''''
' If we already know the density limits, save time
' by setting getdensitylimits=0 ...
' ***-
getdensitylimits = TRUE
' -***
' ... and just set the limits here:
' ***-
MinD = -1!
MaxD = 1!
' -***

'/////////////////////////////////////////////////////
''' everything below here should take care of itself
''' i.e. no adjustable parameters are defined from
''' this point on.
'/////////////////////////////////////////////////////
xlen = xmax - xmin
ylen = ymax - ymin
zlen = zmax - zmin

IF NOT getdensitylimits THEN
        MinDensity = MinD
        MaxDensity = MaxD
        PRINT "Given limits"
ELSE
' Otherwise, find the minimum and maximum of the density in
' our box since densities in the df3 file will have
' to be scaled to integer values from 0 to 255
        x = xmin + xlen * (1 / xres)
        y = ymin + ylen * (1 / yres)
        z = zmin + zlen * (1 / zres)
        MaxDensity = fnDensity(x, y, z)
        MinDensity = MaxDensity

        FOR ix = 1 TO xres
                FOR iy = 1 TO yres
                        FOR iz = 1 TO zres
                           x = xmin + xlen * (ix / xres)
                           y = ymin + ylen * (iy / yres)
                           z = zmin + zlen * (iz / zres)
                           d = fnDensity(x, y, z)
                           IF d > MaxDensity THEN MaxDensity = d
                           IF d < MinDensity THEN MinDensity = d

                        NEXT iz
                NEXT iy
        NEXT ix

END IF
PRINT "minimum:", MinDensity
PRINT "maximum:", MaxDensity
'''''''''''''''''''''''''''''''''
' df3 files are binary
OPEN datafile$ FOR BINARY AS #1
' first two bytes are the x resolution, highbyte lowbyte
bh$ = CHR$(xres \ 256)
bl$ = CHR$(xres MOD 256)
PUT #1, , bh$
PUT #1, , bl$
' next two bytes are the y resolution, highbyte lowbyte
bh$ = CHR$(yres \ 256)
bl$ = CHR$(yres MOD 256)
PUT #1, , bh$
PUT #1, , bl$
' and next two bytes are the z resolution, highbyte lowbyte
bh$ = CHR$(zres \ 256)
bl$ = CHR$(zres MOD 256)
PUT #1, , bh$
PUT #1, , bl$

' the rest of the file is  xres*yres*zres density values.
' I assume they are supposed to be arranged
' as succesive xy planes, calculated row by row
FOR ix = 1 TO xres
        FOR iy = 1 TO yres
                FOR iz = 1 TO zres
                   x = xmin + xlen * (ix / xres)
                   y = ymin + ylen * (iy / yres)
                   z = zmin + zlen * (iz / zres)
                   d = fnDensity(x, y, z)
                   ' the density needs to be an unsigned byte (0 - 255)
                   dens$ = CHR$(INT(255 * (1 / (MaxDensity - MinDensity) * (d - MinDensity))))
                   PUT #1, , dens$
                NEXT iz
        NEXT iy
NEXT ix

CLOSE #1


END

